'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createObjectMethodTransform;

var _package = require('../../package.json');

var _createImport = require('./createImport');

var _createImport2 = _interopRequireDefault(_createImport);

var _isPolyfillPath = require('./isPolyfillPath');

var _isPolyfillPath2 = _interopRequireDefault(_isPolyfillPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createImportIdentifier(file, polyfill) {
  return (0, _createImport2.default)(file, polyfill.split('.').join('_'), `${_package.name}/polyfills/${polyfill.split('.').join('/')}`);
}

function createObjectMethodTransform(polyfill) {
  const [object, property] = polyfill.split('.');

  return {
    CallExpression(path, { file }, { types: t }) {
      const { filename } = file.opts;
      if ((0, _isPolyfillPath2.default)(filename)) {
        return;
      }

      // Special cases: apply & call
      if (t.isMemberExpression(path.node.callee) && t.isMemberExpression(path.node.callee.object) && t.isIdentifier(path.node.callee.property) && ['apply', 'call'].includes(path.node.callee.property.name) && path.node.callee.object.object.name === object && path.node.callee.object.property.name === property) {
        path.replaceWith(t.callExpression(t.memberExpression(createImportIdentifier(file, polyfill), path.node.callee.property), path.node.arguments));
        return;
      }

      // Skip when it is not object.property
      if (!t.isMemberExpression(path.node.callee) || !t.isIdentifier(path.node.callee.object) || !t.isIdentifier(path.node.callee.property) || path.node.callee.object.name !== object || path.node.callee.property.name !== property) {
        return;
      }

      // Happy case, replace with polyfill + import
      path.replaceWith(t.callExpression(createImportIdentifier(file, polyfill), path.node.arguments));
    }
  };
}